package edu.nps.seed.nst

import scala.collection.mutable.Map
import scala.collection.mutable.ListBuffer
import java.io.File
import java.io.PrintWriter


class DataPair(val name: String, val x: AnyVal)
class DataOutput(val header: String, val values: String)


object DataOutputter {
  
  def outputTo(file: File, header: String, values: String) {
	  val writer = new PrintWriter(file)
	  try {
	    //write header
        writer.println(header)
        //write values
        writer.println(values)
	  } finally {
	    writer.close()
	  }
    
  }
  
  def computeStatistics(): List[DataPair] = {
	val values = new ListBuffer[DataPair]
    val statistics =  List(new TotalTestTime(),
                           new TaskStatusCount(TestEvent.taskMap.values.toList),
    					   new ResourceStats(TestEvent.resourceMap.values.toList))
    
	  for (stats <- statistics) { 
	    val results = stats.calculate()
	    for (result <- results) {
	      values += result
	    }
     }
    values.toList
  }
  
  def outputStatistics(filename: String): Unit = {
    val data = computeStatistics()
    val output = toCSV(data)
    
    //echo output to the console
    //print the headers
    println(output.header)
    // print the values
    println(output.values)

    outputTo(new File(filename+".csv"), output.header, output.values)

  }
  
  def toCSV(data: List[DataPair]): DataOutput = {
    val header = new ListBuffer[String]
    val values = new ListBuffer[String]
    
    for (d <- data) {
      header += d.name
      values += d.x.toString
    }
    
    val output = new DataOutput(header.mkString(","),
    						values.mkString(","))
    
    output
  }
}
