package edu.nps.seed.nst
import scala.collection.mutable.Map


object TestEvent extends Simulation {

  val taskMap = Map[String, Task]()
  val resourceMap = Map[String, Resource]()
  var testStartTime: Double = -1.
  var testEndTime: Double = -1.
  var duration_variability: Double = 0.
  
  def initializeTest(scenarioFile: String) = {
	  startTest(0.)
	  readInputFile(scenarioFile)
	  for (task <- taskMap.values) {
	    task.addPredecessors(taskMap)
	    task.addResources(resourceMap)
	    println("Task after add: "+task)
	    Task.scheduleStartTask(task)
	  }
  }
  
  private def readInputFile(scenarioFile: String) {
     val scenario = xml.XML.loadFile(scenarioFile)
     val tasks = scenario \ "tasks"
     val resources = scenario \ "resources"
     duration_variability = (tasks \ "duration_variation").text.toDouble
     for (task <- (tasks \ "task") ) {
//        println("task: "+task.toString)
        var sometask = Task.taskFromXML(task)
        taskMap += (sometask.name -> sometask)
        println("Task: "+sometask)       
       
     }
     for (resource <- (resources \ "resource") ) {
//       println("resource: "+resource.toString)
 		var someresource = Resource.resourceFromXML(resource)
 		resourceMap += (someresource.name -> someresource)
 		println("Resource: "+someresource)       
       
     }

//     println("Scenario:"+scenario.toString)
     println("Number of tasks: "+taskMap.size)
     println("Number of resources: "+resourceMap.size)

  }
  
  
  def runTest() {
    run()
  }
  
  def startTest(time: Double) {
    testStartTime = time
      scheduleEventAtTime(time) {
    	  println("*** simulation started, time = " +
    			  time +" ***")
    	  println(time+",simulation,start")
    
      }

  }
  
  def scheduleEndTestAt(time: Double) {
       scheduleEventAtTime(time) {
		  println(time+",end test,remaining active tasks "+numActiveTasks)  
		  println(time+",end test,remaining busy resources "+numBusyResources)
          clearSchedule()
      }   
  }
  
  def scheduleStartTask(task: Task, time: Double) {
      scheduleEventAtTime(time) {
         task.start()
      }
    
  }
  
  def scheduleStartTaskNow(task: Task) {
      scheduleEventAtTime(currentTime) {
         task.start()
      }
    
  }  
  
  def scheduleEndTask(task: Task, time: Double) {
	  scheduleEventAtTime(time) {
	    task.end()
	  }
  }  
  
  def numActiveTasks(): Int = {
    val activeTasks = 
      for {
        task <- taskMap.values.toList 
        if (task.status == TaskStatus.Active)
      } yield task
    activeTasks.size
  }
  
  def numBusyResources(): Int = {
    val busyResources = 
      for {
        resource <- resourceMap.values.toList
        if (resource.status == ResourceStatus.Busy)
      } yield resource
    busyResources.size
  }
  
  
  def main(args: Array[String]) {
    val reps = args(1).toInt
    for (rep <- 1 to reps) {
      initializeTest(args(0))
//      scheduleEndTestAt(91)
      
	  runTest()
	  testEndTime = currentTime
	  println("************generating statistics**************** ")  
	  DataOutputter.outputStatistics(args(0)+"."+rep)
	  EventLogger.outputEvents(args(0)+"."+rep)      
    }

  }
}
