package edu.nps.seed.nst.pp

import java.io.File
import scala.collection.mutable.ListBuffer
import java.io.PrintWriter


object PostProcessor {
	val excursions = new ListBuffer[Excursion]()
	val allOutput = new ListBuffer[String]()
	val filetypes = Map(".sum.csv" -> ".xml.csv",".tasks.csv" -> ".xml.tasks")
	val header = new ListBuffer[String]()
 
   def postProcess(studyDir: String, studyFileName: String): Unit = {
     println("studyDir: " + studyDir)
     println("studyFileName: " + studyFileName)
     val studyFile = new File(studyDir, studyFileName)
     
     val study = xml.XML.loadFile(studyFile)
     header += "Exc"
     
     val iteration = study \\ "Iteration"
     for (variable <- iteration \ "Variable") {
    	 header += (variable \ "@name").text
     } 	
     
     val excursionsXML = iteration \ "Excursion"
     val outputDir = new File(new File(studyDir), (iteration \\ "MOEDir").text)
     val filestub = (iteration \\ "ExcursionFileStub").text
     println("outputDir: " + outputDir.toString)
     

     //initialize the excursions
     for (excursionXML <- excursionsXML ) {
        val excursion = Excursion.excursionFromXML(excursionXML)
        excursions += excursion
        println("excursion: " + excursion.toString)
       
     }
     
     if ( excursions.size != 0 ) {
	     for (filesuffix <- filetypes.keys) {
	       var hdr = ""
	       for (exc <- excursions) {
	        val results = exc.processOutput(filestub, filetypes(filesuffix),outputDir)
	        hdr = exc.header.stripLineEnd
	        for (line <- results)
	          allOutput += line         
	       }
           header += hdr
	       header.mkString(",") +: allOutput
	       header -= hdr
	       //write to output file
	       writeOutput(allOutput, new File(studyDir, (filestub + filesuffix)))
	       
	       allOutput.clear
	     }       
       
     }
     else {
       println("No excursions found in " + studyFileName)
     }
     

   }
   
   def writeOutput(d: ListBuffer[String], outfile: File): Unit =  {
	  val writer = new PrintWriter(outfile)
	  try {
		  for (line <- d )
			  writer.println(line)
	  } finally {
	    writer.close()
	  }     
     
   }
  
   def checkUsage(args: Array[String]): Boolean = {
     val result = 
       if (args.length == 2) true
       else false
     
     result
   }

   def main(args: Array[String]) {
        if ( checkUsage(args )) {
          println("Success!")
          postProcess(args(0), args(1))
        }
        else {
          println("Failed")
          println("Usage: 2 parameters are required")
        }
          
    	
     
  }
}
