package oldmcdata.agents;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import oldmcdata.analyzers.Analyzer;
import oldmcdata.analyzers.AnalyzerFactory;


/**
 * Title:        AnalyzerAgent<p>
 * Description:  A simple agent that can send  custom messages to another agent.  Based on AgentSender in
 * JADE examples.receivers.AgentSender.<p>
 *Copyright: 2000 - 2005
 * @author Stephen C. Upton
 * @version $Date: 12/4/2000  $ $Revision: 1.0 $
 *
 */

public class AnalyzerAgent extends OldMcDataAgent {
    /**
	 * 
	 */
	private Analyzer analyzer;
    private long timeToWaitInbetweenFileChecks = 10000;
    private long timeToWaitToInitialize = 5000;

    protected void setup() {
        super.setup();
        this.addBehaviour(new InitialBehavior(this));
        System.out.println("this is a test of the AnalyzerAgent arguments list: "
                + "StudyDir: " + studyDirName + " StudyFile: " + studyFileName);
    }


    class InitialBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public InitialBehavior(Agent a) {
            super(a);
        }

        public void action() {
        	myAgent.doWait(timeToWaitToInitialize);
            doInitialization();
            myAgent.addBehaviour(new WaitForAnalysisRequest(myAgent));
            finished = true;
        }

        private void doInitialization() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            analyzer = AnalyzerFactory.getInstance(studyDirName, studyFileName);
            analyzer.setConfigFile(configFileName);
            analyzer.initialize();
        }

        public boolean done() {
            return finished;
        }
    }   // end InitialBehavior


    class WaitForAnalysisRequest extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "DoAnalysis";

        public WaitForAnalysisRequest(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is in waiting for message: " + message);
            System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
//	try {
            MessageTemplate m1 = MessageTemplate.MatchOntology("GEA");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println(msg.toString());
                System.out.println(msg.toString());
//        ContentElement content = myAgent.getContentManager().extractContent(msg);
                //       Object resp = l.get(0);
                //    List msgContent = myAgent.extractContent(msg);
                finished = true;
                myAgent.addBehaviour(new DoAnalysisBehavior(myAgent));

            } else {
                output.println("\nAgent " + myAgent.getLocalName() + " has not received message: " + message);
                block();
            }
//    } // End of try
//    catch (FIPAException fe) {
//            System.err.println("FIPAException in fill/extract content:" + fe.getMessage());
//            System.exit(-1);
//    }
        }

        public boolean done() {
            return finished;
        }


    } // end WaitForAnalysisRequest

    class DoAnalysisBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public DoAnalysisBehavior(Agent a) {
            super(a);
        }

        public void action() {
            if (analyzer.analyze())
                myAgent.addBehaviour(new AnalysisCompleteBehavior(myAgent));
            else {
                myAgent.doWait(timeToWaitInbetweenFileChecks);
                myAgent.addBehaviour(new DoAnalysisBehavior(myAgent));
            }
            finished = true;
        }

        public boolean done() {
            return finished;
        }
    } // end DoAnalysisBehavior

    class AnalysisCompleteBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public AnalysisCompleteBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  has completed Analysis");
            output.println("\nAgent " + myAgent.getLocalName() + "  has completed Analysis");
            sendAnalysisCompleteMessage();
            myAgent.addBehaviour(new WaitForAnalysisRequest(myAgent));
            finished = true;
        }

        private void sendAnalysisCompleteMessage() {
            AID r = new AID();
            r.setLocalName("generator");
            ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
            msg.setSender(myAgent.getAID());
            msg.addReceiver(r);
            msg.setContent("CompletedAnalysis");
            msg.setLanguage("PlainText");
            msg.setOntology("GEA");
            send(msg);
        }

        public boolean done() {
            return finished;
        }
    } // end AnalysisCompleteBehavior


    class WaitForRunsCompletedMessage extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "WaitForRunsCompletedMessage";

        public WaitForRunsCompletedMessage(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
            MessageTemplate m1 = MessageTemplate.MatchOntology("RunsCompleted");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                output.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
                System.out.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
                finished = true;
                output.close();
                myAgent.doDelete();
            } else {
                block();
            }
        }

        public boolean done() {
            return finished;
        }
    }     // end WaitForRunsCompletedMessage
}











