package oldmcdata.analyzers;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXMLHandler;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class AnalyzerFactory {

    public AnalyzerFactory() {
    }


    /** creates instance of Generator based on algorithm name supplied by input file.
     *  Any errors in attempting to construct an instance of this class
     *  will result in the system exit, with a printout of the error trace.
     *  Uses java.lang.reflect methods and classes to construct instance of Generator.
     *
     */
    public static Analyzer getInstance(String studyDirName, String studyFileName) {
        StudyXMLHandler xmlHandler = new DefaultStudyXMLHandler(studyDirName, studyFileName);
        xmlHandler.openStudyXMLDocument();
        String fullyQualifiedClassName = xmlHandler.getAnalyzerAlgorithmClassName();
        Constructor ctor = null;
        Analyzer analyzer = null;
        try {
            Class c = Class.forName(fullyQualifiedClassName);
            ctor = c.getConstructor(null);
            analyzer = (Analyzer) ctor.newInstance(null);
        } catch (ClassNotFoundException cfne) {
            handleException(cfne, "Class: " + fullyQualifiedClassName
                    + " Does Not Exist, or not on CLASSPATH");
        } catch (NoSuchMethodException nsme) {
            handleException(nsme, "Class: " + fullyQualifiedClassName
                    + " has no constructor with 0 arguments");
        } catch (InvocationTargetException ite) {
            handleException(ite, "Can not construct instance of Class: " +
                    fullyQualifiedClassName);
        } catch (IllegalAccessException iae) {
            handleException(iae, "Can not access: " + fullyQualifiedClassName
                    + " - not public");
        } catch (InstantiationException ie) {
            handleException(ie, "Class: " + fullyQualifiedClassName
                    + " can not be constructed as it is an interface or abstract class");
        }
        analyzer.setStudyXMLHandler(xmlHandler);
        return analyzer;
    }

    /** convenience method for consistently handling messages from the
     *  getInstance() method.
     */
    private static void handleException(Exception ex, String message) {
        System.out.println(message);
        ex.printStackTrace(System.out);
        System.exit(-1);
    }

}
