package oldmcdata.analyzers.utils.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import oldmcdata.analyzers.utils.OutputDataFileHandler;
import oldmcdata.analyzers.utils.OutputSchemaBuilder;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class ModelDataProcessor {
    protected final static String DEFAULTDATADELIMITER = ",";
    protected final static int DEFAULTLINESTOSKIP = 0;
    protected BufferedReader fileToProcess;
    protected OutputSchemaBuilder moeFormat;
    protected OutputDataFileHandler outputDataFileHandler;
    protected String delimiter, moe, moePosition;
    protected List selectedData, rundata;
    protected double[] data;
    protected int numberOfReplications;
    protected int linesToSkip = 0;


    public ModelDataProcessor() {
        moeFormat = new OutputSchemaBuilder();
        outputDataFileHandler = new OutputDataFileHandler();
        delimiter = DEFAULTDATADELIMITER;
        selectedData = new ArrayList();
        rundata = new ArrayList();
    }

    public static ModelDataProcessor getInstance(String modelName, String modelVersion) {
        ModelDataProcessor dataProcessor = null;
        Constructor ctor = null;
        StringBuffer composeClassName = new StringBuffer("oldmcdata.analyzers.utils.model.");
        composeClassName.append(modelName);
//		composeClassName.append(modelVersion); wait until we need this one - scu 103102
        composeClassName.append("DataProcessor");
        String fullyQualifiedClassName = composeClassName.toString();

        try {
            Class c = Class.forName(fullyQualifiedClassName);
            ctor = c.getConstructor(null);
            dataProcessor = (ModelDataProcessor) ctor.newInstance(null);
        } catch (ClassNotFoundException cfne) {
            // if can't find one, use this one as default
            dataProcessor = new ModelDataProcessor();
        } catch (NoSuchMethodException nsme) {
            handleException(nsme, "Class: " + fullyQualifiedClassName
                    + " has no constructor with 0 arguments");
        } catch (InvocationTargetException ite) {
            handleException(ite, "Can not construct instance of Class: " +
                    fullyQualifiedClassName);
        } catch (IllegalAccessException iae) {
            handleException(iae, "Can not access: " + fullyQualifiedClassName
                    + " - not public");
        } catch (InstantiationException ie) {
            handleException(ie, "Class: " + fullyQualifiedClassName
                    + " can not be constructed as it is an interface or abstract class");
        }
        dataProcessor.setModelInfo(modelName, modelVersion);
        return dataProcessor;

    }

    public void setModelInfo(String modelName, String modelVersion) {
        moeFormat.setModelInfo(modelName, modelVersion);
    }

    public void setMOE(String moe) {
        this.moe = moe;
    }

    public void setMOEPosition(String moepos) {
        moePosition = moepos;
    }
    
    public void setLinesToSkip(String lines) {
    	if ( lines == null )
    		linesToSkip = DEFAULTLINESTOSKIP;
    	else
    		linesToSkip = Integer.parseInt(lines);
    }    

    public void setConfigFile(File configFile) {
        moeFormat.setConfigFile(configFile);
    }

    public void setOutputFileInfo(File moeDirectory, String moeFileStub) {
        outputDataFileHandler.setOutputFileInfo(moeDirectory, moeFileStub);
    }

    public void setBasecaseFileInfo(File studyDirectory, String basecaseFileName) {
        moeFormat.setBasecaseFileInfo(studyDirectory, basecaseFileName);
    }

    public void setNumberOfReplications(String numberOfReplications) {
        this.numberOfReplications = Integer.parseInt(numberOfReplications);
    }

    public void buildSchema() {
        // simplest schema with one moe-moePosition mapping
        if (moePosition == null)
            throw new RuntimeException("MOE Position has not been specified");
        moeFormat.buildSchema(moe, moePosition);
        delimiter = moeFormat.getDelimiter();
    }

    public double[] getData(String excursionNumber) {
        fileToProcess = outputDataFileHandler.getDataReader(new Integer(excursionNumber));
        if ( fileToProcess == null )
        	return null;
        processData();

        return data;
    }
    
    protected void skipNumberOfLines(int numberOfLinesToSkip) {
        try {
            for (int i = 0; i < numberOfLinesToSkip; i++) {
                String dataLine = fileToProcess.readLine();
            }
        } catch (IOException e) {
            e.printStackTrace(System.out);
        }
    }    

    protected void processData() {
        try {
            skipNumberOfLines(linesToSkip);        	
            for (int i = 0; i < numberOfReplications; i++) {
                String dataLine = fileToProcess.readLine();
                tokenizeDataLine(dataLine);
                selectData();
            }
            convertData();
            fileToProcess.close();
            selectedData.clear();
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

    protected void tokenizeDataLine(String dataLine) {
        StringTokenizer tokenizer = new StringTokenizer(dataLine, delimiter);
        while (tokenizer.hasMoreTokens()) {
            rundata.add(tokenizer.nextToken());
        }
    }

    protected void selectData() {
        int columnSelectedData = moeFormat.getMOEPosition(moe);
        if ( rundata.size() <= columnSelectedData ) {
        	selectedData.add("NaN"); // problem reading output file for some reason
        	rundata.clear();
        	return;
        }
        selectedData.add(rundata.get(columnSelectedData));
        rundata.clear();
    }

    protected void convertData() {
        data = new double[selectedData.size()];
        for (int i = 0; i < data.length; i++) {
        	String currentData = (String) selectedData.get(i);
        	if ( currentData.equalsIgnoreCase("NaN"))
        		data[i] = Double.NaN;
        	else
        		data[i] = Double.parseDouble(currentData);
        }
            
    }

    /** convenience method for consistently handling messages from the
     *  getInstance() method.
     */
    protected static void handleException(Exception ex, String message) {
        System.out.println(message);
        ex.printStackTrace(System.out);
        System.exit(-1);
    }

}