package oldmcdata.evaluators;

import java.io.File;
import java.net.MalformedURLException;
import java.util.List;
import java.util.Map;
import java.util.Random;

import oldmcdata.utils.StudyXML;
import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public abstract class AbstractCommandLineBuilder implements CommandBuilder {
    // tag names from the OldMcData config file
    protected final String NUMBEROFREPLICATIONS = "NumberOfReplications",
    PLAYBACKFILE = "PlaybackFile",
    EXECUTABLE = "executable",
    TYPE = "type",
    VALUE = "value",
    COMMAND = "command",
    PREFIX = "prefix",
    CONDOR = "condor",
    LSF = "LSF",
    ARGUMENTS = "arguments",
    SEED = "Seed";
    protected StudyXMLHandler studyXMLHandler;
    protected File configFile;
    protected List commandLines, seedList, modelNodes, argumentList;
    protected Map argumentMap;
    protected Element configRootElement, configModelElement, seedArgument,
    executableElement, argumentsElement, playbackFileArgument;
    protected String executable, numberOfReplications, startingSeed, modelName;
    protected int numberOfExcursions;
    protected boolean playbacksWanted = false;
    private Random random = new Random();


    public AbstractCommandLineBuilder() {
    }

    public List getCommandLines() {
        return commandLines;
    }

    public abstract void buildCommandLines(File configFile, StudyXMLHandler studyXMLHandler,
                                           Map argumentMap, int numberOfExcursions);
    
    protected void openStudyXML() {
        studyXMLHandler.openStudyXMLDocument();
    }

    protected void openConfigFile() throws MalformedURLException,
            DocumentException {
        SAXReader xmlReader = new SAXReader();
        xmlReader.setStripWhitespaceText(true);
        Document configDoc = xmlReader.read(configFile);
        configRootElement = configDoc.getRootElement();
    }

    protected abstract void setParameters();

    protected void setModelElement() {
        // get the model information from the studyXML
        modelName = studyXMLHandler.getModelName();
        String version = studyXMLHandler.getModelVersion();
		StringBuffer xpathString = new StringBuffer("/OldMcDataConfig/Model[@name='");
		xpathString.append(modelName).append("' and @version='");
		xpathString.append(version).append("']");
		System.out.println("xpath string: " + xpathString.toString());
		XPath xpath = DocumentHelper.createXPath(xpathString.toString());
        Node modelNode = xpath.selectSingleNode(configRootElement);

        if (modelNode == null) {
            throw new RuntimeException("OldMcData FAILED: Can not locate model/version " +
                    modelName + "-" + version + " in the configFile: " +
                    configFile.toString());
        } else {
            System.out.println("model xpath: " + modelNode.getPath());
            configModelElement = (Element) modelNode;
        }



    }

    public Element getExecutableElement(String type) {
        List executables = configModelElement.elements(EXECUTABLE);
        for (int i = 0; i < executables.size(); i++) {
            Element ex = (Element) executables.get(i);
            if (type.equalsIgnoreCase(ex.attributeValue(TYPE)))
                return ex;
        }
        return null;
    }

    protected void setReplicationsInfo() {
        numberOfReplications = studyXMLHandler.getNumberReplicates();
        int numberOfReps = Integer.parseInt(numberOfReplications);
        seedList = studyXMLHandler.getInitialSeeds();
        int numberOfSeeds = seedList.size();
        if (numberOfSeeds < numberOfReps)
            addSeeds(numberOfSeeds, numberOfReps);
    }
    
    private void addSeeds(int numberOfSeeds, int numberOfReps) {
        String lastSeed = (String) seedList.get(numberOfSeeds - 1);
        long seed = new Long(lastSeed).longValue();
        random.setSeed(seed);
        for (int i = numberOfSeeds; i < numberOfReps; i++) {
        	seedList.add(i, Long.toString(random.nextLong()));
        }
        	
    }


    protected void setPlaybacksWanted() {
        Element modelParameters = studyXMLHandler.getModelParametersElement();
        String playbacksWantedString = modelParameters.element(StudyXML.PLAYBACKSWANTED).getTextTrim();
        playbackFileArgument = argumentsElement.element(PLAYBACKFILE);
        if (playbackFileArgument != null) {
            if (playbacksWantedString.equalsIgnoreCase("yes"))
                playbacksWanted = true;
            else  // don't want playbacks, so take off arguments from config file, if exists
                argumentsElement.remove(playbackFileArgument);
        }

    }


}

