package oldmcdata.evaluators.condor;

import java.util.List;

import oldmcdata.evaluators.AbstractEvaluator;
import oldmcdata.evaluators.CommandBuilder;
import oldmcdata.utils.ExcursionFileCreator;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class CondorEvaluator extends AbstractEvaluator {
    private final static String MAKERUNS = "MakeRuns";
    private CondorJobRunner jobRunner;
    private ExcursionFileCreator excursionFileCreator;
    private boolean makeRuns = true;
    private CommandBuilder cmdLineBuilder;

    public CondorEvaluator() {
        super();
        jobRunner = new CondorJobRunner();
        excursionFileCreator = new ExcursionFileCreator();
        cmdLineBuilder = new CondorCommandBuilder();
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + " is in initialize()");
        try {
            configure();
        } catch (Exception e) {
            e.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    public void setMakeRuns(String noe) {
        Boolean b = Boolean.valueOf(noe);
        makeRuns = b.booleanValue();
    }

    private void configure() {
        Element parameters = studyXMLHandler.getEvaluatorParameters();
        Element makeRunsElement = parameters.element(MAKERUNS);
        if (makeRunsElement != null)
            setMakeRuns(makeRunsElement.getTextTrim());
        // else do nothing makeRuns remains true - false setting used for testing
        if (!configFile.exists())
            throw new RuntimeException("OLDMCDATA FAILED:OLDMCDATA configuration file: " +
                    configFile.toString() + " does not exist");
        System.out.println("configFile used: " + configFile.toString());
    }

    public void evaluate() {
        System.out.println(this.getClass().getName() + " is in evaluate()");
        excursionFileCreator.createExcursions(studyXMLHandler);
        cmdLineBuilder.buildCommandLines(configFile, studyXMLHandler,
                excursionFileCreator.getArgumentMap(),
                excursionFileCreator.getNumberOfExcursions());
        if (makeRuns)
            jobRunner.run(cmdLineBuilder.getCommandLines());
        else {
            List commandLines = cmdLineBuilder.getCommandLines();
            System.out.println("CommandLines");
            for (int i = 0; i < commandLines.size(); i++) {
                List commandLine = (List) commandLines.get(i);
                for (int j = 0; j < commandLine.size(); j++) {
                    String arg = (String) commandLine.get(j);
                    System.out.print(arg);
                    System.out.println();
                }
            }
        }
    }

}

