package oldmcdata.generators;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2004
 * Company: EADS Dornier GmbH,  
 * @author Thorsten A. H. Lampe
 * @author Stephen C. Upton
 * @version 1.0
 */

/**
 * Generates the cartesian product from a set of variables which each takes a set of
 * values. This breaks down to generating the Cartesian product of all
 * variables' value sets.
 */
public class CartesianProductGenerator extends AbstractGenerator {
    private final static String DELIMITER = "Delimiter",
                                DEFAULTDELIMITER = ",",
                                VALUE = "Value",
                                INTEGER = "integer",
                                FLOAT = "float";
    private List excursions;
    private String delim;
    private int[] currentIndices;
    private int[] maxIndices;
    private String[][] paramValues;
    private int numberOfExcursions = 1;
    private boolean generatedExcursions = false;

    public CartesianProductGenerator() {
        super();
        delim = DEFAULTDELIMITER;
    }

    /**
     * Initializes this generator.
     */
    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
        translator.initialize();
        configure();
    }

    /**
     * Reads the parameters from the {@link StudyXMLHandler}, making this
     * generator ready for action.
     */
    private void configure() {
        Element parameters = studyXMLHandler.getGeneratorAlgorithmParameters();
        Element delimiter = parameters.element(DELIMITER);
        if (delimiter != null)
            setDelimiter(delimiter.getText());
    }

    /**
     * (Re-)Sets the delimiter with which to separate the different parameters'
     * values in the result strings.
     * @param d the delimiter to use
     */
    public void setDelimiter(String d) {
        delim = d;
    }

    /**
     * Generates the excursion elements from the description given in the
     * <code>Generator</code> element as obtained from the {@link
     * StudyXMLHandler}
     * @return <code>true</code> iff the excursions had already been generated
     *         before
     */
    public boolean generate() {
        if (!generatedExcursions) {
            System.out.println(this.getClass().getName() + ":generate()");
            studyXMLHandler.openStudyXMLDocument();
            createParamValues();
            generateExcursions();

			for (Iterator it = excursions.iterator(); it.hasNext();) {
				System.out.println((String) it.next());
			}
            translator.translate(excursions, studyXMLHandler);
            studyXMLHandler.writeStudyXMLDocument();
            writeRenamedBaseCase();
            generatedExcursions = true;
            return false;
        }
        return true;
    }

    private void createParamValues() {
        List variables = studyXMLHandler.getVariables();
        // Extract all values from the list of variables
        paramValues = new String[variables.size()][];
        int i = 0;
        for (Iterator var = variables.iterator(); var.hasNext(); i++) {
            paramValues[i] = getParamValues((Element) var.next());
        }
        /* Set all indices to 0 and the max indices to the number of values of
         * each variable, respectively
         */
        currentIndices = new int[paramValues.length];
        maxIndices = new int[paramValues.length];
        for (i = 0; i < currentIndices.length; i++) {
            currentIndices[i] = 0;
            maxIndices[i] = paramValues[i].length;
            numberOfExcursions *= maxIndices[i];
        }
    }

    private String[] getParamValues(Element variable) {
        List valueElements = variable.elements(VALUE);
        String[] variableParamValues = new String[valueElements.size()];
        int i = 0;
        for (Iterator it = valueElements.iterator(); it.hasNext(); i++) {
            variableParamValues[i] = ((Element)it.next()).getText();
        }
        return variableParamValues;
    }

    /**
     * Increments the index for every variable.
     */
    private void incrementCurrentIndices() {
        for (int i = 0; i < currentIndices.length; i++) {
            ++currentIndices[i];
            if (currentIndices[i] >= maxIndices[i]) {
                currentIndices[i] = 0;
                continue;
            } else
                break;
        }
    }

    /**
     * Does the actual generation once all precautions have been made.
     */
    private void generateExcursions() {
        excursions = new ArrayList();
        for (int i = 0; i < numberOfExcursions; i++) {
            excursions.add(generateExcursion());
            incrementCurrentIndices();
        }
    }

    private String generateExcursion() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < currentIndices.length; i++) {
            sb.append(paramValues[i][currentIndices[i]]).append(delim);
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

}