package oldmcdata.generators;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXMLHandler;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class GeneratorFactory {

    private GeneratorFactory() {
    }


    /** creates instance of Generator based on algorithm name supplied by study.xml file.
     *  Any errors in attempting to construct an instance of this class
     *  will result in the system exit, with a printout of the error trace.
     *  Uses java.lang.reflect methods and classes to construct instance of Generator.
     *
     */
    public static Generator getInstance(String studyDirName, String studyFileName) {
        StudyXMLHandler xmlHandler = new DefaultStudyXMLHandler(studyDirName, studyFileName);
        xmlHandler.openStudyXMLDocument();
        String fullyQualifiedClassName = xmlHandler.getGeneratorAlgorithmClassName();
        Generator g = null;
        Constructor ctor = null;
        try {
            Class c = Class.forName(fullyQualifiedClassName);
            ctor = c.getConstructor(null);
            g = (Generator) ctor.newInstance(null);
        } catch (ClassNotFoundException cfne) {
            handleException(cfne, "The GeneratorAlgorithm you have specified: " + fullyQualifiedClassName
                    + " Does Not Exist, or not on CLASSPATH");
        } catch (NoSuchMethodException nsme) {
            handleException(nsme, "The GeneratorAlgorithm you have specified: " + fullyQualifiedClassName
                    + " has no constructor with 0 arguments");
        } catch (InvocationTargetException ite) {
            handleException(ite, "Can not construct instance of the GeneratorAlgorithm you have specified: " +
                    fullyQualifiedClassName);
        } catch (IllegalAccessException iae) {
            handleException(iae, "Can not access the GeneratorAlgorithm you have specified: " + fullyQualifiedClassName
                    + " - not public");
        } catch (InstantiationException ie) {
            handleException(ie, "The GeneratorAlgorithm you have specified: " + fullyQualifiedClassName
                    + " can not be constructed as it is an interface or abstract class");
        }
        g.setStudyXMLHandler(xmlHandler);
        return g;
    }


    /** convenience method for consistently handling messages from the
     *  getInstance() method.
     */
    private static void handleException(Exception ex, String message) {
        System.out.println(message);
        ex.printStackTrace(System.out);
        System.exit(-1);
    }

}