/**
 * 
 */
package oldmcdata.utils.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 * @author stephen upton
 * @date 12/23/07
 * 
 */
public class XPathSubstitution {
//	private String fileToChange;
	private String substitutionFile;
	Element substitutionRootElement;
	private ArrayList<File> listOfFilesToChange;
	private List<Element> xpathsToReplaceList;
	private FileHandler fileHandler;
	private XPathReplacementHandler xpathHandler;
	
	
	public XPathSubstitution(String subsFile) {
		substitutionFile = subsFile;
		listOfFilesToChange = new ArrayList<File>();
		xpathHandler = new XPathReplacementHandler();
	}
	
	public void createFileSet() {
		System.out.println("Creating file set.");
		Element filesElement = substitutionRootElement.element("files");
		fileHandler = new FileHandler(filesElement);
		listOfFilesToChange = fileHandler.getFilesToChange();

	}
	
	@SuppressWarnings("unchecked")
	public void makeSubstitutions() {
		System.out.println("Making substitutions.");
		xpathsToReplaceList = substitutionRootElement.elements("XpathsToReplace");
		fileHandler.createOutputDirectory();
		// go through each file and make substitutions based on XPaths
		for (File workingFile : listOfFilesToChange ) {
			makeSubstitutionsSingleFile(workingFile);
		}
	}
	
	private void makeSubstitutionsSingleFile(File workingFile) {
		System.out.println("Making substitutions in file: " + workingFile.toString());
		Document workingDoc = fileHandler.openFileToChange(workingFile);
		
		xpathHandler.setDocumentRoot(workingDoc.getRootElement());
		
		for (Element xpathsToReplace : xpathsToReplaceList ) {
			xpathHandler.processXpathsToReplace(xpathsToReplace);
			
		}
		
		fileHandler.writeModifiedFile(workingDoc, workingFile.getName());
	}



	
	public void openSubstitutionFile() {
		try {
			SAXReader xmlReader = new SAXReader();
			Document fileDoc = xmlReader.read(substitutionFile);
			substitutionRootElement = fileDoc.getRootElement();

		} 
		catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + substitutionFile);
			de.printStackTrace(System.out);
			System.exit(-1);
		}

	}
	
	public static void main(String[] args) {
		System.out.println("Starting subsititutions...");
		String subsFile = args[0];
		
		XPathSubstitution subs = new XPathSubstitution(subsFile);
		subs.openSubstitutionFile();
		subs.createFileSet();
		subs.makeSubstitutions();
		
		System.out.println("Completed subsititutions.");
	}
}
